// Select the needed modules
import { 
  select, 
  csv, 
  scaleLinear,   
  axisLeft,
  extent,
  axisBottom,
  format
} from 'd3';


// Capture in a variable the DOM component 'svg'
const svg = select('svg');

// Capture from the attributes 'height' and 'width' from the 
// DOM component 'svg'
// Use (+) to parse string type to int type.
const height = +svg.attr('height');
const width = +svg.attr('width');

//   d.mpg = +d.mpg;
//   d.cylinders = d.cylinders;
//   d.displacement = d.displacement;
//   d.horsepower = d.horsepower;
//   d.weight = d.weight;
//   d.acceleration = d.acceleration;
//   d.year =  d.year;

// Create 'promise' lamda with rendering instructions
// pass as parameter 'data' array, captured from using 'csv'
const render = data => {
  
  const chartTitle = 'Cars: Horsepower vs Weight';
  // Split data for X axis and Y axis
  const xValue = d => d.horsepower;
  const xAxisLabel = 'Horsepower';
  
  const yValue = d => d.weight;  
  const circleRadius = 8;  
  const yAxisLabel = 'Weight';
  
  // Set Margins following D3 guidelines
  const margin = {top: 50, right: 40, bottom: 60, left: 150}
  
  // Calculating width and height form the chart 'canvas'
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  
  // Define X scale as linear scale and 
  // defining the dataset 'domain' in this case the population field
  // in X axis. Defining the 'range' to which the values will
  // extend, the whole available canvas width 'innerWidth'
  const xScale = scaleLinear()
  	.domain(extent(data,xValue))
    .range([0, innerWidth])
  	.nice();    
  
  // Define Y scale
  // defining the dataset 'domain', in this case the countries field
  // in the Y axis. Defininf the range to whcih the values will
  // extend, the whole available canvas height 'innerHeight'
  const yScale = scaleLinear()
  	.domain(extent(data,yValue))
    .range([0,innerHeight])
  	.nice();  
  
  // Crate a group and render all the chart components inside
  // User transform function to translate (line up) chart componets
  // to the top/left marging
  const g = svg.append('g')
  	.attr('transform',`translate(${margin.left},${margin.top})`)
    
  // Get the buttom axis, formates and define the size
  const xAxis = axisBottom(xScale)  
  	.tickSize(-innerHeight)
  	.tickPadding(15);
  
  // Remove tick and lines from the Y scale axis
  // using DOM class references to '.domain' and '.tick lines'
  const yAxis = axisLeft(yScale).tickSize(-innerWidth);
  
  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove()
  
  yAxisG.append('text')
  		.attr('class', 'axis-label')
  		.attr('y', -40)
  		.attr('x', -innerHeight / 2)
  		.attr('fill', 'black')
  		.attr('transform','rotate(-90)')
  		.attr('text-anchor','middle')
  		.text(yAxisLabel);
   
  
  // Adding the xAxis to a group, that way I can remove the domain line
  const xAxisG = g.append('g').call(xAxis)
  	.attr('transform',`translate(0,${innerHeight})`);
  
  // Remove the domain axis
  xAxisG.selectAll('.domain').remove();
  
  
  // Add title to X axis, assigning class attribute to use a reference in CSS
  xAxisG.append('text')
  		.attr('class', 'axis-label')
  		.attr('y', 45)
  		.attr('x', innerWidth / 2)
  		.attr('fill', 'black')
  		.text(xAxisLabel);
  
  // Add bars and passing axis and data values promises as parameters
  g.selectAll('circle').data(data)
  	.enter().append('circle')
  		.attr('cy', d => yScale(yValue(d)))
  		.attr('cx', d => xScale(xValue(d)))
  	  .attr('r',circleRadius);  
  
  // Add graph title
  g.append('text')
  	.attr('class', 'title')
  	.attr('y', -10)
  	.text(chartTitle);
};

csv('https://vizhub.com/curran/datasets/auto-mpg.csv').then(data => {
 data.forEach(d => {
   d.mpg = +d.mpg;
   d.cylinders = +d.cylinders;
   d.displacement = +d.displacement;
   d.horsepower = +d.horsepower;
   d.weight = +d.weight;
   d.acceleration = +d.acceleration;
   d.year = +d.year;
 }); 
 render(data);
});